/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.model;

import hu.midori.kosmos.model.util.IdentifiedEntity;
import hu.midori.kosmos.model.util.ZoomableImageLocator;

import java.util.List;
import java.util.Map;

/**
 * Wraps a <i>JIRA</i> project.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class JiraProject extends IdentifiedEntity {
    /** Project name. */
    private String name;
    /** JIRA URL. */
    private String url;
    /** Project key. */
    private String key;
    /** JIRA URL. */
    private String projectUrl;
    /** Description. */
    private String description;
    /** Project leader name. */
    private String lead;

    /* Number of issues per type related to this project. */
    private int openIssues;
    private int codingInProgressIssues;
    private int reopenedIssues;
    private int resolvedIssues;
    private int closedIssues;

    /* Number of open issues per priorty related to this project. */
    private int blockerOpenIssues = 0;
    private int criticalOpenIssues = 0;
    private int majorOpenIssues = 0;
    private int minorOpenIssues = 0;
    private int trivialOpenIssues = 0;
    private int optionalOpenIssues = 0;

    /** Open-issues-per-assignee sorted by issues in descending order. */
    private List<Map.Entry<String,Integer>> openIssuesPerAssignee;

    /** Locator of the issues-per-status chart image. */
    private ZoomableImageLocator issuesPerStatusChartLocator;
    /** Locator of the open-issues-per-priority chart image. */
    private ZoomableImageLocator openIssuesPerPriorityChartLocator;
    /** Locator of the open-issues-per-assignee chart image. */
    private ZoomableImageLocator openIssuesPerAssigneeChartLocator;

    /**
     * By design this should be private to avoid instantiation
     * without discriminator, but Hessian needs default ctor.
     */
    public JiraProject() {
        super(null);
    }

    public JiraProject(String name, String url, String key, String projectUrl, String description, String lead,
        int openIssues, int codingInProgressIssues, int reopenedIssues, int resolvedIssues, int closedIssues,
        int blockerOpenIssues, int criticalOpenIssues, int majorOpenIssues, int minorOpenIssues, int trivialOpenIssues, int optionalOpenIssues,
        List<Map.Entry<String,Integer>> openIssuesPerAssignee,
        ZoomableImageLocator issuesPerStatusChartLocator, ZoomableImageLocator openIssuesPerPriorityChartLocator, ZoomableImageLocator openIssuesPerAssigneeChartLocator) {
        super(name + url + key + projectUrl);

        this.name = name;
        this.url = url;
        this.key = key;
        this.projectUrl = projectUrl;
        this.description = description;
        this.lead = lead;

        this.openIssues = openIssues;
        this.codingInProgressIssues = codingInProgressIssues;
        this.reopenedIssues = reopenedIssues;
        this.resolvedIssues = resolvedIssues;
        this.closedIssues = closedIssues;

        this.blockerOpenIssues = blockerOpenIssues;
        this.criticalOpenIssues = criticalOpenIssues;
        this.majorOpenIssues = majorOpenIssues;
        this.minorOpenIssues = minorOpenIssues;
        this.trivialOpenIssues = trivialOpenIssues;
        this.optionalOpenIssues = optionalOpenIssues;

        this.openIssuesPerAssignee = openIssuesPerAssignee;

        this.issuesPerStatusChartLocator = issuesPerStatusChartLocator;
        this.openIssuesPerPriorityChartLocator = openIssuesPerPriorityChartLocator;
        this.openIssuesPerAssigneeChartLocator = openIssuesPerAssigneeChartLocator;
    }

    public String getName() {
        return name;
    }

    public String getUrl() {
        return url;
    }

    public String getKey() {
        return key;
    }

    public String getProjectUrl() {
        return projectUrl;
    }

    public String getDescription() {
        return description;
    }

    public String getLead() {
        return lead;
    }

    public int getOpenIssueAssignees() {
        return getOpenIssuesPerAssignee().size();
    }

    public int getOpenIssues() {
        return openIssues;
    }

    public int getCodingInProgressIssues() {
        return codingInProgressIssues;
    }

    public int getReopenedIssues() {
        return reopenedIssues;
    }

    public int getResolvedIssues() {
        return resolvedIssues;
    }

    public int getClosedIssues() {
        return closedIssues;
    }

    public int getBlockerOpenIssues() {
        return blockerOpenIssues;
    }

    public int getCriticalOpenIssues() {
        return criticalOpenIssues;
    }

    public int getMajorOpenIssues() {
        return majorOpenIssues;
    }

    public int getMinorOpenIssues() {
        return minorOpenIssues;
    }

    public int getTrivialOpenIssues() {
        return trivialOpenIssues;
    }

    public int getOptionalOpenIssues() {
        return optionalOpenIssues;
    }

    public List<Map.Entry<String,Integer>> getOpenIssuesPerAssignee() {
        return openIssuesPerAssignee;
    }

    public ZoomableImageLocator getIssuesPerStatusChartLocator() {
        return issuesPerStatusChartLocator;
    }

    public ZoomableImageLocator getOpenIssuesPerPriorityChartLocator() {
        return openIssuesPerPriorityChartLocator;
    }

    public ZoomableImageLocator getOpenIssuesPerAssigneeChartLocator() {
        return openIssuesPerAssigneeChartLocator;
    }

    public int getTotalIssues() {
        return getOpenIssues() + getCodingInProgressIssues() + getReopenedIssues() + getResolvedIssues() + getClosedIssues();
    }

    public String toString() {
        return String.format("[%s] %s", key, name);
    }
}
