/*
 * Kosmos.
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package hu.midori.kosmos.server.util;

import java.io.ByteArrayInputStream;
import java.net.URL;
import java.util.List;

import net.sf.saxon.Configuration;
import net.sf.saxon.dom.DocumentBuilderFactoryImpl;
import net.sf.saxon.dom.DocumentWrapper;
import net.sf.saxon.om.NodeInfo;
import net.sf.saxon.query.DynamicQueryContext;
import net.sf.saxon.query.StaticQueryContext;
import net.sf.saxon.query.XQueryExpression;
import net.sf.saxon.trans.XPathException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.tidy.Tidy;

/**
 * Utility methods for web crawling.
 *
 * @author <a href="mailto:aron.gombas@midori.hu">Aron Gombas</a>
 * @version $Id$
 */
public class WebCrawlingUtils {
    private static Log log = LogFactory.getLog(WebCrawlingUtils.class);

    /** This class should never be instantiated. */
    protected WebCrawlingUtils() { }

    /** Parses an XML document from the given string. */
    public static Document parseStringDom(String xmlString) throws Exception {
        log.debug(String.format("Parsing DOM from %d long string...", xmlString.length()));

        //return DocumentBuilderFactoryImpl.class.newInstance().newDocumentBuilder().parse(new StringBufferInputStream(xmlString));
        return DocumentBuilderFactoryImpl.class.newInstance().newDocumentBuilder().parse(new ByteArrayInputStream(xmlString.getBytes()));
    }

    /** Downloads an XML document from the given URL and returns it as DOM. */
    public static Document downloadXmlDom(URL url) throws Exception {
        log.debug(String.format("Downloading DOM from \"%s\"...", url.toString()));

        Document dom = DocumentBuilderFactoryImpl.class.newInstance().newDocumentBuilder().parse(url.openStream());
        log.debug(String.format("DOM top-level children: %d", dom.getChildNodes().getLength()));

        return dom;
    }

    /** Downloads and tidies up an HTML document from the given URL and returns it as DOM. */
    public static Document downloadHtmlDom(URL url) throws Exception {
        log.debug(String.format("Downloading HTML DOM from \"%s\"...", url.toString()));

        Tidy tidy = new Tidy();
        tidy.setQuiet(true);
        tidy.setShowWarnings(false);
        tidy.setXmlTags(false);
        Document dom = tidy.parseDOM(url.openStream(), null);
        log.debug(String.format("DOM top-level children: %d", dom.getChildNodes().getLength()));

        return dom;
    }

    /**
     * Returns the first node with the given attribute value from the given list or
     * <code>null</code> if not found.
     */
    public static Node findDomNodeByAttribute(NodeList nodes, String attribName, String attribValue) {
        for(int i = 0; i < nodes.getLength(); i++) {
            Node node = nodes.item(i);
            Node attrib = node.getAttributes().getNamedItem(attribName);
            if(attrib == null)
                continue;
            if(attrib.getNodeValue().indexOf(attribValue) != -1)
                return node;
        }

        return null;
    }

    /** Runs an XQuery on the given DOM and returns the full result. */
    public static List runXQuery(Document dom, String query) throws XPathException {
        Configuration configuration = new Configuration();
        StaticQueryContext staticContext = new StaticQueryContext(configuration);
        XQueryExpression queryExpression = staticContext.compileQuery(query);
        DynamicQueryContext dynamicContext = new DynamicQueryContext(configuration);
        dynamicContext.setContextNode(new DocumentWrapper(dom, "", configuration));

        return queryExpression.evaluate(dynamicContext);
    }

    /** Runs an XQuery on the given DOM and returns a single int as result. */
    public static int runXQueryInt(Document dom, String query) throws XPathException {
        return Integer.parseInt(runXQueryString(dom, query));
    }

    /** Runs an XQuery on the given DOM and returns a single String as result. */
    public static String runXQueryString(Document dom, String query) throws XPathException {
        List list = runXQuery(dom, query);
        if(list.size() != 1)
            throw new IllegalStateException(String.format("Unable to retrieve a single value, found %d", list.size()));

        return ((NodeInfo)list.get(0)).getStringValue();
    }

    /**
     * Eliminates the empty items from a scraped value string to make
     * the tokenizer happy. E.g. <code>||xxx|</code> will be transformed to <code> | |xxx| </code>.
     */
    public static String eliminateEmptyValues(String value) {
        if(value.charAt(0) == '|')
            value = " " + value;
        value = value.replaceAll("\\|\\|\\|", "| | |");
        value = value.replaceAll("\\|\\|", "| |");
        return value;
    }
}
